#!/usr/bin/python

"""
Alerts windows control thread
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"
import platform
import os, sys
from ctypes import *
import time
import threading
from tkinter import *
from tkinter.simpledialog import SimpleDialog
from gcn_ztcs import *
from gcn_bta import *
from gcn_show import show_sky_init, show_sky_start, tel_obj_quit


tm=None     # timeout thread
root=None   # main Tk window
dq=None     # current dialog window
last=None   # last dialog message
lock=False  # dialog locking flag
warn=False  # warning dialog flag

play_wav=False
fwav=None
pa=None

def boom():
    global fwav, pa, play_wav
    # Empty handler
    def py_error_handler(filename, line, function, err, fmt):
#        print('')
        pass
    if not play_wav:
        return
    try:
        import pyaudio, wave
        play_wav=True
    except ImportError:
        play_wav=False
        return
# Define our error handler type
    if platform.system() == "Linux":
# Linux Only? but what about Windows? need not?:
        ERROR_HANDLER_FUNC = CFUNCTYPE(None, c_char_p, c_int, c_char_p, c_int, c_char_p)
        c_error_handler = ERROR_HANDLER_FUNC(py_error_handler)
        asound = cdll.LoadLibrary('libasound.so.2') # edit this for real libasound name!
# Set error handler
        asound.snd_lib_error_set_handler(c_error_handler)

    if not os.access("alarm.wav", os.F_OK):
        print("Can't access alarm.wav!")
        play_wav=False
        return
    else:
        if not os.access("alarm.wav", os.R_OK):
            print("Can't access alarm.wav!")
            play_wav=False
            return
#    chunk = 1024
    if fwav==None:
        try:
            fwav = wave.open(r"./alarm.wav","rb")
        except wave.Error:
            print("Can't open alarm.wav!")
            play_wav=False
            return            
    else:
        fwav.rewind()
    if pa==None:
        pa = pyaudio.PyAudio()
    stream = pa.open(format = pa.get_format_from_width(fwav.getsampwidth()),
                    channels = fwav.getnchannels(),
                    rate = fwav.getframerate(),
                    output = True)
    try:
        data = fwav.readframes(fwav.getnframes())
        if len(data) > 0:
            stream.write(data)
    except wave.Error:
        print("Can't read frames from alarm.wav!")
        fwav.close()
        play_wav=False
        
#    data = fwav.readframes(chunk)
#    while len(data) > 0:
#        stream.write(data)
#        data = fwav.readframes(chunk)
    
    stream.stop_stream()
    stream.close()
    del stream

telname="Zeiss" # Name of SAO's telescope: 'BTA' or 'Zeiss'
telhost="ann"   # Network name of TCS computer 
user = None     # Zeiss TCS user (level>=2 for 'Send'; level>=3 for 'Slew')
passwd = None   # Password (for BTA: level>=2 for 'Send'; level>=4 for 'Slew')
is_bta = False  # default - Zeiss-1000
level = 0       # TCS access level

def alerts_init(tname=None,thost=None,uname=None,pswd=None,audio=False):
    global telname,telhost,user,passwd,is_bta,level,play_wav
    if tname!=None: telname=tname
    if thost!=None: telhost=thost
    if uname!=None: user=uname
    if pswd!=None: passwd=pswd
    if telname.upper()=="BTA": is_bta=True

    show_sky_init(telname,telhost,user,passwd)

    if audio:
        try:
            import pyaudio, wave
            play_wav=True
        except ImportError:
            play_wav=False
            print("No PyAudio package? Can't play wav-signals!")

cmd = "wait"
new_ra = 0.
new_dec = 0.
new_name = "?"
new_msg = " "
new_tout = 0.
new_err =0.
thr_mon = None

def alerts_mon_start():
    global root, cmd, new_ra,new_dec,new_name,new_msg,new_tout,new_err
    global telname,telhost,user,passwd,is_bta,level
    if root == None:
        try:
            root = Tk()
        except TclError:
            print("Can't start Tk to show window!?")
            sys.stdout.flush()
            return None   
        root.geometry('+0+0')
        f = Frame(root)
        f.pack(fill=BOTH,expand=1)
        l = Label(f,text="G C N\nfor\n"+telname)
        l.pack(fill=BOTH, expand=1)
        time.sleep(0.2)
    if is_bta:
        if bta_init(thost=telhost,pswd=passwd):
            level=bta_level()
    else:
        if ztcs_init(thost=telhost,uname=user,pswd=passwd):
            level=ztcs_level()
    print(time.ctime(time.time()), "Alerts monitoring thread start...")
    sys.stdout.flush()
    while cmd != "stop":
#        print(time.ctime(time.time()), "Alerts.mon cmd="+cmd)
        sys.stdout.flush()
        if cmd == "gcn" or cmd == "warn":
            if is_bta and level==0:
                if bta_init(): 
                    level=bta_level()
            elif level==0:
                if ztcs_init():
                    level=ztcs_level()
            if cmd == "gcn":
                cmd = "wait"
                alert_gcn(new_ra,new_dec,new_name,new_msg,new_tout,new_err)
            else:
                cmd = "wait"
                alert_warn(new_msg,new_tout)
#            tm = threading.Timer(0.5,root.quit) 
#            tm.start()
#            root.mainloop()
#            del tm
        else:
            time.sleep(0.5)
#        print(time.ctime(time.time()), "Alerts_mon cmd="+cmd)
        sys.stdout.flush()
    root.destroy()
    print(time.ctime(time.time()), "Alerts monitoring thread stop...")
    sys.stdout.flush()
    return None

def alerts_mon_stop():
    global cmd
    cmd = "stop"
#    print(time.ctime(time.time()),"alerts_mon_stop()")
    sys.stdout.flush()
    

# Show dialog window for GCN RA/Dec coordinates
gcn_time = 0.0  # previous GRB time (sec)
gcn_err = 77.   # previous GRB error box (degr)
def show_alert_gcn(ra,dec,name,msg,tout,err):
    global thr_mon, gcn_time, gcn_err, dq, last, lock
    global cmd, new_ra,new_dec,new_name,new_msg,new_tout,new_err
    t = time.time()
    if gcn_time>0.01 and t-gcn_time < 2.0:
        time.sleep(0.2)
        print(time.ctime(t), "Can't show window for '"+msg+"', because '"+last+"' window just already started...")
        return None
    gcn_time = t
    if thr_mon == None or not thr_mon.is_alive():
        thr_mon = threading.Timer(0.1, alerts_mon_start)
        thr_mon.start()
    if lock:
        if dq != None:
            alert_deiconify()
        if err >= gcn_err:
            print(time.ctime(time.time()), "Can't show new GRB window, because dialog '"+last+"' working...")
            return None
        if dq != None:
            tel_obj_quit()
            dq.done(dq.cancel)
            dq.root.update()
#            print(time.ctime(time.time()), 'Stop dialog "'+last+'"')
        sys.stdout.flush()
        while cmd != "wait" and time.time()-t < 2.0:
            time.sleep(0.2)
    new_ra = ra
    new_dec = dec
    new_name = name
    new_msg = msg
    new_tout = tout
    new_err = err
    cmd = "gcn"
    return thr_mon

def alert_deiconify():
    global root, dq
    if root.state() == "iconic":
        print("DeIconify the Root window...")
        root.deiconify()
    if dq!=None and isinstance(dq,SimpleDialog):
        if dq.root.state() == "iconic":
            print("DeIconify the Alert window...")
            dq.root.deiconify()

def show_gcn():
    global dq, new_ra,new_dec,new_name
    show_sky_start(new_ra*15., new_dec, new_name, dq.root)

def alert_gcn(ra,dec,name,msg,tout,err):
    global root, dq, last, lock, warn, gcn_err, play_wav
    global telname,is_bta,level
    if lock: time.sleep(1.0)
        
    def alert_stop():
        alert_deiconify()
        if dq!=None and isinstance(dq,SimpleDialog):
#            print(time.ctime(time.time()), 'Timeout! SimpleDialog done...')
            dq.done(dq.cancel)
            dq.root.update()

    lock=True
    warn = False    
#    print(time.ctime(time.time()), 'Start GCN dialog window for "'+msg+'"')
    sys.stdout.flush()
    serr = "ErrBox=%g(arcmin)" % (err*60.)
    obj = "RA="+hour2hms(ra)+" Dec="+degr2dms(dec)+" Name="+name   
    quest = "Are You want to send them to "+telname+" TCS?"
    if level<=1:
        butnames = ["Ok"]
        quest = "Have not rights to send them to "+telname+" TCS!"
    elif level<=2 or (is_bta and level<=3): 
        butnames = ["Cancel","Send to "+telname+" TCS"]
    else:
        butnames = ["Cancel","Only Send to "+telname,"Send and Slew"]
        quest = "Are You want to start "+telname+" slewing?"
    dq = SimpleDialog(master=root,
                      title=msg,
                      text=msg+"\n"+serr+"\n"+obj+"\n"+quest,
                      buttons=butnames,
                      cancel=0,
                      default=1)
    bts = Button(dq.frame, text ="Show@Sky", command=show_gcn)
    bts.pack(side=LEFT, fill=BOTH, expand=1)
#    tm = threading.Timer(tout, dq.done, [dq.cancel])
    dq.root.after(int(900), alert_deiconify)
    id = dq.root.after(int(tout*1000), alert_stop)
#    print(time.ctime(time.time()), 'Timeout '+str(tout)+'sec for "'+msg+'"' )
    last=msg
    gcn_err = err
#    tm.start()
    if play_wav: boom()
    try:
        dq.go()
    except TclError:
        print("Tcl/Tk Error!?")
#        pass
    tel_obj_quit()
    dq.root.update()
    nrep = dq.num
    dq.root.after_cancel(id)
    del dq
    dq=None
#    if isinstance(tm, threading.Thread):
#        if tm.is_alive(): tm.cancel()
#        del tm
    if nrep == 1:       # Send RA/Dec coordinates to TCS
        print(time.ctime(time.time()), "Send "+obj+" to "+telname+" TCS")
        if is_bta:
            if not bta_send_to(ra,dec,objname=name):
                alert_warn("Can't send "+name+" to BTA TCS!",10.0)
        else:
            if not ztcs_send_to(ra,dec,objname=name):
                alert_warn("Can't send "+name+" to Zeiss TCS!",10.0)
    elif nrep == 2:     # Slew telescope to RA/Dec coordinates
        print(time.ctime(time.time()), "Slew telescope to "+obj)
        if is_bta:
            if not bta_slew_to(ra,dec,objname=name):
                alert_warn("Can't slew BTA to "+name+"!",10.0)
        else:
            if not ztcs_slew_to(ra,dec,objname=name):
                alert_warn("Can't slew Zeis to "+name+"!",10.0)
    else:
        print(time.ctime(time.time()), "Cancel "+obj)
    sys.stdout.flush()
    lock=False
    
# Show warning dialog window 
warn_time = 0.0  # previous GRB time (sec)
def show_alert_warn(msg,tout):
    global thr_mon, warn_time, last, lock, warn, dq
    global cmd, new_ra,new_dec,new_name,new_msg,new_tout,new_err
    t = time.time()
    if warn_time>0.01 and t-warn_time < 2.0:
        time.sleep(0.2)
        print(time.ctime(t), "Can't show window for '"+msg+"', because '"+last+"' window just already started...")
        return None
    warn_time = t
    if thr_mon == None or not thr_mon.is_alive():
        thr_mon = threading.Timer(0.1, alerts_mon_start)
        thr_mon.start()
    if lock and dq != None:
        alert_deiconify()
    if lock and warn:
        if dq != None:
            dq.done(dq.cancel)
            dq.root.update()
            print(time.ctime(time.time()), 'Stop dialog "'+last+'"')
        sys.stdout.flush()
        while cmd != "wait" and time.time()-t < 2.0:
            time.sleep(0.2)
    elif lock:
        print(time.ctime(time.time()), "Can't show warning dialog window, because dialog '"+last+"' working...")
        return None
    new_ra = 0.
    new_dec = 0.
    new_name = "?"
    new_msg = msg
    new_tout = tout
    new_err = 0.
    cmd = "warn"
    return thr_mon

def alert_warn(msg,tout):
    global root, dq, last, lock, warn, gcn_err, play_wav
    def alert_stop():
        alert_deiconify()
        if dq!=None and isinstance(dq,SimpleDialog):
#            print(time.ctime(time.time()), 'Timeout! SimpleDialog done...')
            dq.done(dq.cancel)
            dq.root.update()
    lock = True
    warn = True
#    print(time.ctime(time.time()), 'Start warning dialog "'+msg+'"')
    sys.stdout.flush()
    dq = SimpleDialog(master=root,
                      title=msg,
                      text=msg+"\nIt's a warning only! Continue...",
                      buttons=["Ok"],
                      cancel=0,
                      default=0)
#    tm = threading.Timer(tout, dq.done, [dq.cancel])
    dq.root.after(int(900), alert_deiconify)
    id = dq.root.after(int(tout*1000), alert_stop)
#    print(time.ctime(time.time()), 'Timeout '+str(tout)+'sec for "'+msg+'"' )
    last=msg
    gcn_err = 77. 
#    tm.start()
#    if play_wav: boom()
    try:
        dqret=dq.go()
        dq.root.update()
#        print(time.ctime(time.time()), "Warn.SimpleDialog="+str(dqret))
    except TclError:
        print("Tcl/Tk Error!?")
#        pass
    dq.root.after_cancel(id)
    del dq
    dq=None
#    if isinstance(tm, threading.Thread):
#        if tm.is_alive(): tm.cancel()
#        del tm
    lock = False
    warn = False    

#-------------------------------------------
# Test code
if __name__ == "__main__":
#    play_wav=True
#    boom()
#    time.sleep(1.0)
#    boom()
#    time.sleep(1.0)
#    sys.exit(0)
    
    alerts_init(tname="BTA",pswd="ooper",audio=True)
#    alerts_init(tname="Zeiss",uname="user",pswd="ztcs",audio=True)
    thr1=show_alert_warn("Warn.Message.Test1",8.0)
    if thr1 != None: print(time.ctime(time.time()),"show_alert_warn(Test1) "+thr1.getName()+" running...")
    sys.stdout.flush()
    time.sleep(9.0)
    thr3=show_alert_gcn(8.5,10.5,"GCN_123456","GCN test1" ,8.0,5.1)
    if thr3 != None: print(time.ctime(time.time()),"show_alert_gcn(GCN test1) "+thr3.getName()+" running...")
    sys.stdout.flush()
    time.sleep(9.0)
    thr2=show_alert_warn("Warn.Message.Test2",8.0)
    if thr2 != None: print(time.ctime(time.time()),"show_alert_warn(Test2) "+thr2.getName()+" running...")
    sys.stdout.flush()

#    time.sleep(2.1)
#    alerts_mon_stop()
#    time.sleep(1.1)
#    sys.exit(0)
    
    time.sleep(9.0)
    thr4=show_alert_gcn(8.5,10.5,"GCN_654321","GCN test2" ,4.0,4.0)
    if thr4 != None: print(time.ctime(time.time()),"show_alert_gcn(GCN test2) "+thr4.getName()+" running...")
    sys.stdout.flush()
    time.sleep(1.1)
    alerts_mon_stop()
    if thr4 != None: 
        print(time.ctime(time.time()),thr4.getName()+" running...")
        while thr4.is_alive():
            time.sleep(1.0)
            print(time.ctime(time.time()),"Wait..."+thr4.getName())
            alerts_mon_stop()
    elif thr3 != None: 
        print(thr3.getName()+" running...")
        while thr3.is_alive():
            time.sleep(1.0)
            print("Wait..."+thr3.getName())
            alerts_mon_stop()
    print(time.ctime(time.time()),"Main thread finished...")
