#!/usr/bin/python
"""
Start of GCN-client application.
Initialize and start alerts windows control thread from gcn_alerts.py
Use payload handler from gcn_client.py 
to start network listen in pygcn (package by Leo Singer <leo.singer@ligo.org>)
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import os, sys, time
import gcn
from gcn_client import gcn_handler, gcn_use_tests
from gcn_alerts import alerts_init, show_alert_warn

gcn_host1 = "209.208.78.170"    # VOEvent 1.1 (27nov2011) Atlantic.net cloud service( don't working as GCN now???)
gcn_host2 = "50.116.49.68"      # VOEvent 1.1 (29jan2012) Linode.com cloud service
gcn_host3 = "68.169.57.253"     # VOEvent 2.0 (15apr2012) eApps.com cloud service
gcn_host4 = "45.58.43.186"      # VOEvent 2.0 (17nov2015) Atlantic server
gcn_host5 = "104.245.39.13"     #??? VOEvent 2.0 (02oct2019) Atlantic4 (don't working???)
gcn_port = 8099
gcn_proxy=None

is_bta = False
telname = "Zeiss"
tcshost = "ztcs"
tcsuser = None
passwd = None
logname = "gcn_client.log"
use_tests = False
play_wav = False

def show_help():
    print("""GCN/TAN client monitor usage:
  gcn_monitor.py [-h] [Tel=...] [Host=...] [User=...] [Pass=...] [Proxy=...] [Test=Yes] [Audio=Yes]
\tHelp or -h - show this text
\tTel  - telescope name: 'BTA' or 'Zeiss' (default - Zeiss)
\tHost - TCS site name to connect to
\tUser - for Zeiss TCS only (access level>=2 for 'Send'; >=3 for 'Slew')
\tPass - users password for Zeiss TCS or
\t     access level password for BTA (>=2 for 'Send'; >=4 for 'Slew')
\tProxy - name or address of relay computer where gcn_proxy.py was started
\tTest - utilize GCN Test-messages for debugging (default - No)
\tAudio- play WAV-signal on GCN alert (if PyAudio installed,default - No)
""")

if len(sys.argv)==1:
            show_help()
            sys.exit(0)
  
# Check program name (for "BTA" or "Zeiss"-defaul) 
if sys.argv[0].upper().find("BTA") >= 0:
    is_bta = True
    telname = "BTA"
    tcshost = "tb"
    
# Check program name (for "Test") 
if sys.argv[0].lower().find("test") >= 0:
    use_tests = True

if len(sys.argv) > 1:
# Parse command line arguments, if they are present
    for i in range(1,len(sys.argv)):
        l = sys.argv[i]
        if i==1 and (l.lower().find("-h") >= 0 or l.lower().find("help") >= 0):
            show_help()
            sys.exit(0)
        if l.count('=')==1:
            nv = l.split('=')
            nam = str(nv[0])
            val = str(nv[1])
            if   nam.lower().startswith("tel"):
                telname = val
                if telname.upper() == "BTA": is_bta = True 
            elif nam.lower().startswith("host"): tcshost = val.lower()
            elif nam.lower().startswith("user"): tcsuser = val
            elif nam.lower().startswith("pas"): passwd = val
            elif nam.lower().startswith("pro"): gcn_proxy = val.lower()
            elif nam.lower().startswith("test") and val.lower().startswith("y"):
                use_tests = True
            elif nam.lower().startswith("aud") and val.lower().startswith("y"):
                play_wav = True
if is_bta:
    telname = "BTA"
    alerts_init(telname,tcshost,pswd=passwd,audio=play_wav)
    if gcn_proxy:
        gcn_host = gcn_proxy
    else:        
        if tcshost.startswith("ann"):
            gcn_host = gcn_host2
        else:
            gcn_host = gcn_host4
    logname = "gcn_client_"+telname.lower()+".log"
else:
    telname = "Zeiss"
    alerts_init(telname,tcshost,tcsuser,pswd=passwd,audio=play_wav)
    if gcn_proxy:
        gcn_host = gcn_proxy
    else:        
        if tcshost.startswith("ann"):
            gcn_host = gcn_host2
        else:
            gcn_host = gcn_host3
    logname = "gcn_client_"+telname.lower()+".log"
    telname = "Zeiss-1000"
show_alert_warn("Start of GCN/TAN client\n for "+telname+" TCS\n(TCP:"+gcn_host+":"+str(gcn_port)+")",10)
time.sleep(1.0)
sys.stdout.flush()
if os.access(logname, os.F_OK):
    sys.stdout = open(logname,'a')
    print(" ")
    print("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-")
    print(" ")
else:
    sys.stdout = open(logname,'w')
print(time.ctime(time.time()), "Start GCN/TAN for "+telname+" TCS (TCP:"+gcn_host+":"+str(gcn_port)+")")
if use_tests:
    gcn_use_tests()
    print("Use GCN Test-messages for debugging too...")
sys.stdout.flush()
gcn.listen(host=gcn_host, port=gcn_port, handler=gcn_handler)
