#!/usr/bin/python

"""
Retransmitter of VOEvent protocol for three GCN servers
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import sys, time
import threading
import socket

gcn_host1 = "209.208.78.170"    # VOEvent 1.1 on Atlantic.net cloud service
gcn_host2 = "50.116.49.68"      # VOEvent 1.1 on Linode.com cloud service
gcn_host3 = "68.169.57.253"     # VOEvent 2.0 on eApps.com cloud service
gcn_host4 = "45.58.43.186"      # VOEvent 2.0 new (24 Nov 2015) server
gcn_port = 8099
#gcn_hosts = (gcn_host1, gcn_host2, gcn_host3, gcn_host4)
gcn_hosts = (gcn_host4, gcn_host3, gcn_host2, gcn_host1)
n_hosts = len(gcn_hosts)
gcn_busy = [False, False, False, False, True]
localaddr=''
alive_timeout=150   # seconds elapse without any packets from the server
max_reconnect_timeout=600

def tcp_trans(n,rsock,wsock):
    global gcn_busy

    name = threading.current_thread().getName()
    print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") Start")
    sys.stdout.flush()
    while gcn_busy[n]:
        try:
            data = rsock.recv(16384)
            if len(data) == 0:
                gcn_busy[n]=False
                print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") connection closed by peer?")
            else:
                wsock.sendall(data)
                del data
        except socket.timeout:
            gcn_busy[n]=False
            print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") socket timed out")
        except socket.error:
            gcn_busy[n]=False
            print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") socket error")
    try:
        rsock.shutdown(socket.SHUT_RDWR)
        rsock.close()
    except socket.error:
#        print name,"could not close r-socket"
        pass
    else:
        print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") r-connection closed")
    try:
        wsock.shutdown(socket.SHUT_RDWR)
        wsock.close()
    except socket.error:
#        print name,"could not close w-socket"
        pass
    else:
        print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") w-connection closed")
    print(time.ctime(time.time()), "Ch"+str(n)+"("+name+") Stop")
    sys.stdout.flush()

if __name__ == '__main__':
    busy_timeout = 10
    i=0
    l_sock = socket.socket()   # listen for clients socket
    try:
        l_sock.bind((localaddr, gcn_port))
        print(time.ctime(time.time()), "GCN Proxy listen on '%s':%d" % (localaddr, gcn_port))
        sys.stdout.flush()
        l_sock.listen(0)
        while True:
            (c_sock, addr) = l_sock.accept()   # new client socket
            reconnect_timeout = 5
            for j in range(n_hosts):
#                i = (i+1)%n_hosts
                i = j;
                if not gcn_busy[i]: break
            if gcn_busy[i]:
                print("No free channels for new clients!")
                try:
                    c_sock.close()
                except socket.error:
                    print("could not close connection socket")
                else:
                    print(time.ctime(time.time()), "Client connection closed... Can retry in %d seconds" % (busy_timeout))
                time.sleep(busy_timeout)
                if busy_timeout < max_reconnect_timeout:
                    busy_timeout += 10
                continue
            busy_timeout = 10
            while not gcn_busy[i]:
                try:
                    s_sock = socket.socket()   # socket to GCN server
                    try:
                        s_sock.settimeout(alive_timeout)
                        s_sock.connect((gcn_hosts[i], gcn_port))
                        print(time.ctime(time.time()), "Ch%d connected to %s:%d" % (i, str(addr), gcn_port))
                        print(time.ctime(time.time()), "Ch%d connected to %s:%d" % (i, gcn_hosts[i], gcn_port))
                        sys.stdout.flush()
                        gcn_busy[i] = True
                    except socket.error:
                        try:
                            gcn_busy[i] = False
                            s_sock.close()
                        except socket.error:
                            print("could not close connection socket") 
                        else:
                            print(time.ctime(time.time()), "Ch"+str(i)+") connection to "+gcn_hosts[i]+" closed")
                    time.sleep(1.0)
                    if gcn_busy[i]:
                        proc1 = threading.Thread(target=tcp_trans,name="c2s",args=(i,c_sock,s_sock))
                        proc2 = threading.Thread(target=tcp_trans,name="s2c",args=(i,s_sock,c_sock))
                        proc1.start()
                        proc2.start()
                        break
                    else:
                        print("could not connect to %s:%d, will retry in %d seconds" % (gcn_hosts[i], gcn_port, reconnect_timeout))
                        time.sleep(reconnect_timeout)
                        if reconnect_timeout < max_reconnect_timeout:
                            reconnect_timeout += 10
                        for j in range(n_hosts):
                            i = (i+1)%n_hosts
                            if not gcn_busy[i]: break
                except socket.error:
                    print("could not connect to %s:%d, will retry in %d seconds" % (gcn_hosts[i], gcn_port, reconnect_timeout))
                    time.sleep(reconnect_timeout)
                    if reconnect_timeout < max_reconnect_timeout:
                        reconnect_timeout += 10
                    for j in range(n_hosts):
                        i = (i+1)%n_hosts
                        if not gcn_busy[i]: break

    except KeyboardInterrupt:
        print(time.ctime(time.time()), "Keyboard Interrupt")
    except socket.error:
        print("socket error")
    finally:
        for i in range(n_hosts):
            gcn_busy[i] = False
        l_sock.close()
        print(time.ctime(time.time()), "GCN Proxy Stop")
