#!/usr/bin/python

"""
Access to Zeiss-1000 TCS through it XML-RPC server
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import sys, re, socket
from xmlrpc.client import ServerProxy, Boolean, ProtocolError, Fault, Error

latitude=43.6548
server_url = None  # networked XML-RPC server
usr="user"
psw="ztcs"
host="ann"
port=8088
server = None
level = 0

def ztcs_ok():
    "Internal function"
    global server, level
    return isinstance(server,ServerProxy) and level>0

def ztcs_init(thost=None,uname=None,pswd=None):
    """ Create an object to represent our XML-RPC server
    and try to connect to it.
    """
    global server,server_url,usr,psw,host,port,level
    if thost!=None: host=thost
    if uname!=None: usr=uname
    if pswd!=None: psw=pswd
    if not ztcs_ok():
        if server_url == None:
            if host.find('.')>=0:
                server_url = "http://"+usr+":"+psw+"@"+host+":"+str(port)
            else:
                server_url = "http://"+usr+":"+psw+"@"+host+".sao.ru:"+str(port)
        if not isinstance(server,ServerProxy):
            server = ServerProxy(server_url)
        print("Try to connect to Zeiss TCS (URL: "+server_url+")")
        level=0
        try:
            print("Zeiss TCS Network Connection: "+str(server.Server.echo()))
            info = server.Server.info()
            n = info.rfind("Level=")
            if n>0:
               level = int(info[n+6])
        except socket.error as serr:
            print("Zeiss TCS Network Error: %s" % serr.strerror)
            try:
                server.__close()
            finally:
                del server
                server = None
                return False
    return True
def ztcs_level():
    global level
    return level

def ztcs_status():
    "Get Zeiss TCS current state"
    global server,level
    if not ztcs_init():
        return "Off"
    if level<1:
        print("User's level too low!")
        return "Error"
    try:
        repl = server.Telescope.getStatusString("Status")
        if repl.startswith("Status="):
            return str(repl.split('=')[1])
        print("TCS server Error: %s" % repl)
        return "Error"
    except socket.error as serr:
        print("Network Error: %s" % serr.strerror)
    except ProtocolError as err:
        print("A protocol error %d: %s" % err.errcode, err.errmsg)
    except Fault as err:
         print("Fault %d: %s" % err.faultCode, err.faultString)
    except Error as v:
        print("ERROR", v) 
    try:
        server.__close()
    finally:
        del server
        server = None
        return "Error"

def ztcs_is_on(stat=None):
    "Zeiss TCS and telescope are ready to work?"
    if stat == None:
        stat = ztcs_status()
    return  stat.startswith("Stop") or stat.startswith("Mov") or stat.startswith("Slew") or stat.startswith("Track")

def ztcs_send_to(ra,dec,epoch=2000.0,objname=None):
    """ Send RA/Dec coordinates to Zeiss TCS
    RA in hours (hh.hhhhh) Dec in degrees (dd.ddddd)
    Return True if coordinates is Ok
    """
    global server,level
    if not ztcs_init():
        return False
    if level<2:
        print("User's level %d - too low!" % level)
        return False
    if dec>90. or dec <= latitude-90.:
        print("Unreachable Declination %f" % dec)
        return False
    if ra>=24.: ra -= 24.
    if ra<0: ra += 24.
    reqst = "RA="+str(ra)+" Decl="+str(dec)+" Epoch="+str(epoch)
    if objname != None:
        reqst = reqst + " Name="+objname
    try:
        ok = server.Telescope.setupObject(reqst)
        if ok:
           print(reqst," - Ok")
        else:
           print(reqst," - currently unreachable!")
        return ok
    except socket.error as serr:
        print("Network Error: %s" % serr.strerror)
    except ProtocolError as err:
        print("A protocol error %d: %s" % err.errcode, err.errmsg)
    except Fault as err:
         print("Fault %d: %s" % err.faultCode, err.faultString)
    except Error as v:
        print("ERROR", v) 
    try:
        server.__close()
    finally:
        del server
        server = None
        return False

def ztcs_start_slew():
    "Start telescope slewin"
    global server,level
    if not ztcs_init():
        return False
    if level<3:
        print("User's level %d - too low!" % level)
        return False
    if not ztcs_is_on():
       print("Can't slew! Zeiss TCS is \"Off\"!")
       return False
    try:
        ok = server.Telescope.slewToObject()
        if ok:
           print("Start slewing...")
        else:
           print("Slew to object - failed!")
        return ok
    except socket.error as serr:
        print("Network Error: %s" % serr.strerror)
    except ProtocolError as err:
        print("A protocol error %d: %s" % err.errcode, err.errmsg)
    except Fault as err:
         print("Fault %d: %s" % err.faultCode, err.faultString)
    except Error as v:
        print("ERROR", v) 
    try:
        server.__close()
    finally:
        del server
        server = None
        return False
        
def ztcs_slew_to(ra,dec,epoch=2000.0,objname=None):
    """ Slew telescope to RA/Dec coordinates.
    RA in hours (hh.hhhhh) Dec in degrees (dd.ddddd)
    Returns True if telescope and coordinates are Ok and slewing begins.
    """
    if not ztcs_is_on():
       print("Can't slew! Zeiss TCS is \"Off\"!")
       return False
    if ztcs_send_to(ra,dec,epoch=epoch,objname=objname):
        return ztcs_start_slew()
    return False

#--------------------------------------------------------------
# Sexagesimal format functions
def hour2hms(hour):
    "Convert hours floats to a sexagesimal text format."
    if hour>=24.0: hour -= 24.0
    if hour<0.0: hour += 24.0
    h = int(hour)
    m = int((hour-h)*60.)
    s = ((hour-h)*60.-m)*60.
    if s>59.99: s=59.99
    return ("%02d" % h)+':'+("%02d" % m)+':'+("%05.2f" % s)

def degr2dms(degr):
    "Convert degrees floats to a sexagesimal text format."
    sign = '+'
    if degr<0.:
        sign = '-'
        degr = -degr
    if degr>=360.0: degr -= 360.0
    d = int(degr)
    m = int((degr-d)*60.)
    s = ((degr-d)*60.-m)*60.
    if s>59.9: s=59.9
    return sign+("%02d" % d)+':'+("%02d" % m)+':'+("%04.1f" % s)

def hms2hour(hms):
    "Parse sexagesimal text format and return float hours"
    ts = hms.split(':')
    if len(ts)==1:
        return float(ts[0])
    elif len(ts)==2:
        return int(ts[0])+float(ts[1])/60.
    else:
        return int(ts[0])+float(ts[1])/60.+float(ts[2])/3600.

def dms2degr(dms):
    "Parse sexagesimal text format and return float degrees"
    sign = 1
    ts = dms.split(':')
    d = int(float(ts[0]))
    if dms.startswith('-'):
        sign = -1
        d = -d
    if len(ts)==1:
        return float(ts[0])
    elif len(ts)==2:
        return sign*(d+float(ts[1])/60.)
    else:
        return sign*(d+float(ts[1])/60.+float(ts[2])/3600.)

#-------------------------------------------
# Test code
import time
if __name__ == "__main__":
    zst = ztcs_status()
    print("Status: "+zst)
    if ztcs_is_on(zst):
        print("Zeiss TCS is \"On\"")
    else:
        print("Zeiss TCS is \"Off\"")
    ztcs_send_to(15.0,10.0,objname="Test_GCN1")
    time.sleep(6.0)
    ztcs_slew_to(13.0,10.0,objname="Test_GCN2")
    time.sleep(6.0)
    zst = ztcs_status()
